% Solving the discrete adjoint Fokker-Planck equation.
function f1 = adj_fok_pl_adi_2(u_1,u_2,f,kt)

clear u_11 u_12 u_13 u_21 u_22 u_23

[a,b, m, h, x, y, N, t, dt, eta, rho] = parameters(kt);



f1(:,:,:) = zeros(m+1,m+1,N);

tr1 = traj2(t(N),1);
tr2 = [0 0];
for i = 1:m+1
    for j = 1:m+1
        f1(i,j,N) = -200*V2(x(i),y(j),tr1);
    end
end


%fprintf('Running the adjoint model\n');
for k = N-1:-1:1
    k;
    t1 = t(k) + dt/2;
    % Interpolating at half time step
    u_111 = (u_1(k+1)+u_1(k))/2;
    u_131 = u_1(k);
    u_121 = u_1(k+1);
    
    u_212 = (u_2(k+1)+u_2(k))/2;
    u_232 = u_2(k);
    u_222 = u_2(k+1);
    
    tr11 = traj1(t1,1);
    tr12 = traj1(t1+dt/2,1);
    tr21 = [0 0];
    tr22 = [0 0];
    % Boundary condition.
    for i=1:m+1
        f2(i,1) = 0;    
        f2(i,m+1) = 0;
        f2(1,i) = 0;
        f2(m+1,i) = 0;
    end
    % Implicit difference in x-direction
    for j = 2:m
        A = sparse(m-1,m-1);b =zeros(m-1,1);
        for i = 2:m
                     
            K_1 = (1-delta_1(t1,x(i)+h/2,y(j),h,u_111,u_212))*B_3(t1,x(i)+h/2,y(j),h,u_111,u_212) + (1/h)*C_1(t1,x(i)+h/2,y(j),h);
            K_2 = (1/h)*C_1(t1,x(i)+h/2,y(j),h) - delta_1(t1,x(i)+h/2,y(j),h,u_111,u_212)*B_3(t1,x(i)+h/2,y(j),h,u_111,u_212);
            K_3 = (1-delta_1(t1,x(i)-h/2,y(j),h,u_111,u_212))*B_3(t1,x(i)-h/2,y(j),h,u_111,u_212) + (1/h)*C_1(t1,x(i)-h/2,y(j),h);
            K_4 = (1/h)*C_1(t1,x(i)-h/2,y(j),h) - delta_1(t1,x(i)-h/2,y(j),h,u_111,u_212)*B_3(t1,x(i)-h/2,y(j),h,u_111,u_212);
            
            L_1 = (1-delta_2(t1,x(i),y(j)+h/2,h,u_121,u_222))*B_4(t1,x(i),y(j)+h/2,h,u_121,u_222) + (1/h)*C_2(t1,x(i),y(j)+h/2,h);
            L_2 = (1/h)*C_2(t1,x(i),y(j)+h/2,h) - delta_2(t1,x(i),y(j)+h/2,h,u_121,u_222)*B_4(t1,x(i),y(j)+h/2,h,u_121,u_222);
            L_3 = (1-delta_2(t1,x(i),y(j)-h/2,h,u_121,u_222))*B_4(t1,x(i),y(j)-h/2,h,u_121,u_222) + (1/h)*C_2(t1,x(i),y(j)-h/2,h);
            L_4 = (1/h)*C_2(t1,x(i),y(j)-h/2,h) - delta_2(t1,x(i),y(j)-h/2,h,u_121,u_222)*B_4(t1,x(i),y(j)-h/2,h,u_121,u_222);
            
            b(i-1) = 2*f1(i,j,k+1)/dt + (1/h)*(L_2*f1(i,j+1,k+1)-(L_2+L_3)*f1(i,j,k+1)+L_3*f1(i,j-1,k+1))-rho^2*f(i,j,k);
            
            if(i==2)
                A(i-1,i) = -K_2/h;
                A(i-1,i-1) = 2/dt+K_2/h+K_3/h;
            elseif(i==m)
                A(i-1,i-2) = -K_3/h;
                A(i-1,i-1) = 2/dt+K_2/h+K_3/h;
            else
                A(i-1,i-1) = 2/dt+K_2/h+K_3/h;
                A(i-1,i) = -K_2/h;
                A(i-1,i-2) = -K_3/h;              
            end
        end
        unew = A\b;
        for i=1:m-1,
            f2(i+1,j) = unew(i);
        end
    end
    
    
    %Boundary condition.
    for i=1:m+1,
        f1(i,1,k) = 0;
        f1(i,m+1,k) = 0;
        f1(1,i,k) = 0;
        f1(m+1,i,k) = 0;
    end
    % Implicit difference in y-direction
    for i = 2:m
        A = sparse(m-1,m-1);b =zeros(m-1,1);
        for j = 2:m
            
            K_1 = (1-delta_1(t1,x(i)+h/2,y(j),h,u_111,u_212))*B_3(t1,x(i)+h/2,y(j),h,u_111,u_212) + (1/h)*C_1(t1,x(i)+h/2,y(j),h);
            K_2 = (1/h)*C_1(t1,x(i)+h/2,y(j),h) - delta_1(t1,x(i)+h/2,y(j),h,u_111,u_212)*B_3(t1,x(i)+h/2,y(j),h,u_111,u_212);
            K_3 = (1-delta_1(t1,x(i)-h/2,y(j),h,u_111,u_212))*B_3(t1,x(i)-h/2,y(j),h,u_111,u_212) + (1/h)*C_1(t1,x(i)-h/2,y(j),h);
            K_4 = (1/h)*C_1(t1,x(i)-h/2,y(j),h) - delta_1(t1,x(i)-h/2,y(j),h,u_111,u_212)*B_3(t1,x(i)-h/2,y(j),h,u_111,u_212);
            
            L_1 = (1-delta_2(t1,x(i),y(j)+h/2,h,u_131,u_232))*B_4(t1,x(i),y(j)+h/2,h,u_131,u_232) + (1/h)*C_2(t1,x(i),y(j)+h/2,h);
            L_2 = (1/h)*C_2(t1,x(i),y(j)+h/2,h) - delta_2(t1,x(i),y(j)+h/2,h,u_131,u_232)*B_4(t1,x(i),y(j)+h/2,h,u_131,u_232);
            L_3 = (1-delta_2(t1,x(i),y(j)-h/2,h,u_131,u_232))*B_4(t1,x(i),y(j)-h/2,h,u_131,u_232) + (1/h)*C_2(t1,x(i),y(j)-h/2,h);
            L_4 = (1/h)*C_2(t1,x(i),y(j)-h/2,h) - delta_2(t1,x(i),y(j)-h/2,h,u_131,u_232)*B_4(t1,x(i),y(j)-h/2,h,u_131,u_232);
            
            
            b(j-1) = 2*f2(i,j)/dt + (1/h)*(K_2*f2(i+1,j)-(K_2+K_3)*f2(i,j)+K_3*f2(i-1,j))-V1(x(i),y(j),tr11)-rho^2*f(i,j,k);
            
            if(j==2)
                A(j-1,j) = -L_2/h;
                A(j-1,j-1) = 2/dt+L_2/h+L_3/h;
            elseif(j==m)
                A(j-1,j-2) = -L_3/h;
                A(j-1,j-1) = 2/dt+L_2/h+L_3/h;
            else
                A(j-1,j-1) = 2/dt+L_2/h+L_3/h;
                A(j-1,j) = -L_2/h;
                A(j-1,j-2) = -L_3/h;              
            end
        end
              
        unew = A\b;
        for j=1:m-1,
            f1(i,j+1,k) = unew(j);
        end
       
    end

end
