% The algorithm to perform NCG optimization.

function [u11, u12, u21, u22] = ncg(rho1, rho2,kt)

[a,  b, m, h, x, y, Nt, t, dt, nu, eta, rho] = parameters(kt);

u_max = 5;
u_min = -5;

%Initial Guess
u11 = zeros(1,Nt);
u12 = zeros(1,Nt);
u21 = zeros(1,Nt);
u22 = zeros(1,Nt);

if (rho > 1)
    for k = 1:Nt
        u11(k) = 0.1*sin(pi/3*t(k));%0.25;
        u12(k) = 0.1*sin(2*pi/3*t(k));%0.25;
        u21(k) = 0.1*sin(pi/3*t(k));%0.25;
        u22(k) = 0.2*sin(pi/3*t(k));%0.25;
    end
else
    for k = 1:Nt
        u11(k) = 0.0001*sin(pi*t(k));
        u12(k) = 0.0001*sin(2*pi*t(k));
        u21(k) = 0.0001*sin(pi*t(k));
        u22(k) = 0.0002*sin(pi*t(k));
    end
end


f1 = fok_pl_adi(u11,u12,rho1,kt);
f2 = fok_pl_adi_2(u21,u22,rho2,kt);
p1 = adj_fok_pl_adi(u11,u12,f2,kt);
p2 = adj_fok_pl_adi_2(u21,u22,f1,kt);


% Evaluating the gradient with the initial guess.
g11 = grad_j(u11,u12,f1,p1,1,kt);
g12 = grad_j(u11,u12,f1,p1,2,kt);
g21 = grad_j(u21,u22,f2,p2,1,kt);
g22 = grad_j(u21,u22,f2,p2,2,kt);

d11 = -g11;
d12 = -g12;
d21 = -g21;
d22 = -g22;

fprintf('\n\nStarting the NCG\n\n');

k = 1;
k_max = 15;
tol = 10^-5;
alpha1 = 0.5;

while(k < k_max && sqrt(inner_g(g11,g12,g11,g12)+inner_g(g21,g22,g21,g22)) > tol)
    alpha = lin_search(f1,u11,u12,d11,d12,g11,g12,rho1, f2,u21,u22,d21,d22,g21,g22,rho2,kt);
    
    if(alpha < eps)
        fprintf('k=%d, alpha= %.4e,J(u) = %.2e\n',k,alpha1, J(f1,f2,u11,u12,u21,u22,kt));
        break;
    else
        u11 = u11 + alpha*d11;
        u12 = u12 + alpha*d12;
        u21 = u21 + alpha*d21;
        u22 = u22 + alpha*d22;
        
        %Projection Method
        
        u11 = max(u_min, min(u11,u_max));
        u12 = max(u_min, min(u12,u_max));
        u21 = max(u_min, min(u21,u_max));
        u22 = max(u_min, min(u22,u_max));
        
        
        f1 = fok_pl_adi(u11,u12,rho1,kt);
        f2 = fok_pl_adi_2(u21,u22,rho2,kt);
        p1 = adj_fok_pl_adi(u11,u12,f2,kt);
        p2 = adj_fok_pl_adi_2(u21,u22,f1,kt);
        
        gn11 = grad_j(u11,u12,f1,p1,1,kt);
        gn12 = grad_j(u11,u12,f1,p1,2,kt);
        gn21 = grad_j(u21,u22,f2,p2,1,kt);
        gn22 = grad_j(u21,u22,f2,p2,2,kt);
        
        % Evaluating beta.
        beta = (inner_g(gn11,gn12,gn11,gn12)+inner_g(gn21,gn22,gn21,gn22))/(inner_g(d11,d12,gn11-g11,gn12-g12)+inner_g(d21,d22,gn21-g21,gn22-g22));
        
               
        % Descent directions
        d11 = -gn11 + beta*d11;
        d12 = -gn12 + beta*d12;
        d21 = -gn21 + beta*d21;
        d22 = -gn22 + beta*d22;
        
    end
    
    % Evaluating gradient for next iteration
    g11 = gn11;
    g12 = gn12;
    g21 = gn21;
    g22 = gn22;
    fprintf('k=%d, alpha= %.4e, J(u) = %.2e\n',k,alpha, J(f1,f2,u11,u12, u21, u22,kt));
    k = k + 1;
end
